#include <string.h>
#include <json-c/json_object.h>

#include "config.h"
#include "netex.h"
#include "strutil.h"

#define SEPARATOR_CHAR '.'


int config_set(CONFIGURATION* config, const char* key, const char* value)
{
    if (config == NULL)
    {
        WARN("Parameter 'config' is NULL!");
        return -1;
    }
    SPLIT_STR* splitted_key = splstr(key, SEPARATOR_CHAR);
    if (splitted_key == NULL || splitted_key->count == 0)
        return -1;
    if (config->json == NULL)
        config->json = json_object_new_object();
    json_object* temp_obj = NULL;
    for (int i = 0; i != splitted_key->count; i++)
    {
        json_object* current_obj;
        if (temp_obj == NULL)
            current_obj = config->json;
        else
            current_obj = temp_obj;

        // If object with the key already exists, set the temp variable to the found object and continue the loop. (The found object will be used next time in the loop.)
        if (json_object_object_get_ex(current_obj, splitted_key->delimited[i], &temp_obj))
            continue;

        if (i == splitted_key->count - 1) // Last key (set value)
        {
            json_object* value_object = json_object_new_string(value);
            if (json_object_object_add(current_obj, splitted_key->delimited[i], value_object) != 0)
                ERROR("Failed to set value '%s'", value);
            break;
        }

        json_object* new_json = json_object_new_object();
        if (json_object_object_add(current_obj, splitted_key->delimited[i], new_json) != 0)
        {
            ERROR("Failed to set new JSON object!");
            break;
        }
        temp_obj = new_json;
    }
    strsplfree(splitted_key);
    return 0;
}

char* config_get(const CONFIGURATION* config, const char* key)
{
    if (config == NULL || config->json == NULL)
        return NULL;
    SPLIT_STR* splitted = splstr(key, SEPARATOR_CHAR);
    if (splitted == NULL || splitted->count == 0)
        return NULL;
    json_object* tmp_json = config->json;
    char* found_value = NULL;
    for (int i = 0; i != splitted->count; i++)
    {
        json_object* current_json_object = json_object_new_object();
        if (json_object_object_get_ex(tmp_json, splitted->delimited[i], &current_json_object))
            tmp_json = current_json_object;
        else // No key found
        {
            strsplfree(splitted);
            return NULL;
        }
    }
    if (tmp_json != NULL)
        found_value = strdup(json_object_get_string(tmp_json));
    strsplfree(splitted);
    return found_value;
}

char* config_get_default(const char* key, const char* default_value)
{
    return "";
}